/*
 *     JavaFile.java  Jan 30, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.w3c.dom.*;
import java.util.*;
import java.io.*;

public class JavaFile extends JavaElement {
    private String name = null;
    private String packageName = null;
    private ArrayList importNames = new ArrayList();  // String
    private ArrayList classes = null;  // Node
    private ArrayList interfaces = null;  // Node
    private HashMap fqnMap = null;  // String/String

    protected JavaFile() {
    }

    public JavaFile(Node node) {
        super(node);
    }

    public String getName() {
        if (name == null) {
            name = elem.getAttribute(JXConstants.PathAttr);
        }
        return name;
    }

    public String getBaseName() {
        String name = getName();
        int index = name.lastIndexOf(File.separator);
        if (index != -1) {
            return name.substring(index + 1);
        }
        return name;
    }

    public String getDirName() {
        String name = getName();
        int index = name.lastIndexOf(File.separator);
        if (index != -1) {
            return name.substring(0, index);
        }
        return "";
    }

    public String getJarName() {
        return elem.getAttribute(JXConstants.JarAttr);
    }

    public boolean equals(JavaFile jf) {
        return jf != null && getName().equals(jf.getName());
    }

    public Node getPackageNode() {
        return getFirstChildNode(elem, JXConstants.PackageElem);
    }

    public ArrayList getImportNodes() {
        return getChildNodes(elem, JXConstants.ImportElem);
    }

    public ArrayList getClassNodes() {
        if (classes == null) {
            classes = getDescendantNodes(elem, JXConstants.ClassElem);
        }
        return classes;
    }

    public JavaClassList getAllClasses() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getClassNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }

        return jclasses;
    }

    public int getNumClass() {
        return getClassNodes().size();
    }

    public ArrayList getInterfaceNodes() {
        if (interfaces == null) {
            interfaces = getDescendantNodes(elem, JXConstants.InterfaceElem);
        }
        return interfaces;
    }

    public JavaClassList getAllInterfaces() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getInterfaceNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }
        return jclasses;
    }

    public JavaClassList getAllClassesAndInterfaces() {
        JavaClassList jclasses = new JavaClassList();

        jclasses.addAll(getAllClasses());
        jclasses.addAll(getAllInterfaces());
        return jclasses;
    }

    public ArrayList getClassAndInterfaceNodes() {
        ArrayList nodes = new ArrayList();

        nodes.addAll(getClassNodes());
        nodes.addAll(getInterfaceNodes());
        return nodes;
    }

    public String getPackageName() {
        if (packageName == null) {
            packageName = getQName((Element)getPackageNode());
        }
        return packageName;
    }

    public ArrayList getImportNames() {
        if (importNames.size() == 0) {
            Iterator it = getImportNodes().iterator();
            while (it.hasNext()) {
                String importName = getQName((Element)it.next());
                if (importName.length() != 0) {
                    importNames.add(importName);
                }
            }
            importNames.add("java.lang.*");
        }
        return importNames;
    }
    
    public JavaClass getJavaClass(String cname) {
        Iterator it = getClassAndInterfaceNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());

            if (jc.getFqn().equals(cname)) {
                return jc;
            }
        }
        return null;
    }

    public String getFileNameOfClass(String fqn) {
        setHashMap();
        return (String)fqnMap.get(fqn);
    }

    public ArrayList getUsingFiles() {
        setHashMap();

        ArrayList files = new ArrayList();
        Iterator it = fqnMap.values().iterator();
        while (it.hasNext()) {
            String path = (String)it.next();
            path = path.substring(path.indexOf(File.pathSeparator) + 1);
            files.add(path);
        }
        return files;
    }

    private void setHashMap() {
        if (fqnMap == null) {
            fqnMap = new HashMap();

            NodeList nl = elem.getElementsByTagName(JXConstants.FqnMapElem);
            for (int i = 0; i < nl.getLength(); i++) {
                Element e = (Element)nl.item(i);

                String fqn = e.getAttribute(JXConstants.FqnAttr);
                String path = e.getAttribute(JXConstants.PathAttr);
                String jar = e.getAttribute(JXConstants.JarAttr);
                fqnMap.put(fqn, jar + File.pathSeparator + path);
            }
        }
    }

    public ArrayList getAncestorFiles() {
        return collectFiles(getAncestorClasses());
    }

    public JavaClassList getAncestorClasses() {
        JavaClassList classes = new JavaClassList();

        Iterator it = getAllClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            classes.addAll(jc.getAncestors());
        }

        it = getAllInterfaces().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            classes.addAll(jc.getAncestors());
        }
        return classes;
    }

    public ArrayList getDescendantFiles() {
        return collectFiles(getAncestorClasses());
    }

    public JavaClassList getDescendantClasses() {
        JavaClassList classes = new JavaClassList();

        Iterator it = getAllClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            classes.addAll(jc.getDescendants());
        }

        it = getAllInterfaces().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            classes.addAll(jc.getDescendants());
        }
        return classes;
    }

    private ArrayList collectFiles(JavaClassList classes) {
        ArrayList files = new ArrayList();

        Iterator it = classes.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            String fileName = jc.getJavaFile().getName();
            if (!files.contains(fileName)) {
                files.add(fileName);
            }
        }
        return files;
    }

    public void print(StringBuffer buf) {
        print("", buf);
    }

    public void print(String indent, StringBuffer buf) {
        buf.append(indent + "FILE: " + getName() + "\n");
        if (getPackageName().length() != 0) {
            buf.append(indent + "PACKAGE: " + getPackageName() + "\n");
        } else {
            buf.append(indent + "PACKAGE: (none) \n");
        }
        Iterator it = getImportNames().iterator();
        while (it.hasNext()) {
            buf.append(indent + "IMPORT: " + (String)it.next() + "\n");
        }
        printAllClasses(indent, buf);
    }

    public void printAllClasses(String indent, StringBuffer buf) { 
        Iterator it = getClassNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jc.print(indent, buf);
        }

        it = getInterfaceNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jc.print(indent, buf);
        }
    }
}
